;; Per-directory local variables for GNU Emacs 23 and later.

((nil . ((fill-column . 78)
	 (tab-width   .  8)))
 (c-mode . ((c-file-style . "gnu")
	    (indent-tabs-mode . nil)))
 (scheme-mode
  .
  ((indent-tabs-mode . nil)
   (eval . (put 'mcron-error 'scheme-indent-function 1)))))
